import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/main_module/models/space_connection_model.dart';
import 'package:syncrow_web/pages/space_management_v2/main_module/models/space_reorder_data_model.dart';
import 'package:syncrow_web/pages/space_management_v2/main_module/painters/spaces_connections_arrow_painter.dart';
import 'package:syncrow_web/pages/space_management_v2/main_module/widgets/create_space_button.dart';
import 'package:syncrow_web/pages/space_management_v2/main_module/widgets/space_card_widget.dart';
import 'package:syncrow_web/pages/space_management_v2/main_module/widgets/space_cell.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/domain/models/community_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/domain/models/space_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/presentation/bloc/communities_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/presentation/communities_tree_selection_bloc/communities_tree_selection_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/presentation/helpers/space_details_dialog_helper.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

class CommunityStructureCanvas extends StatefulWidget {
  const CommunityStructureCanvas({
    required this.community,
    required this.selectedSpace,
    super.key,
  });

  final CommunityModel community;
  final SpaceModel? selectedSpace;

  @override
  State<CommunityStructureCanvas> createState() => _CommunityStructureCanvasState();
}

class _CommunityStructureCanvasState extends State<CommunityStructureCanvas>
    with SingleTickerProviderStateMixin {
  final Map<String, Offset> _positions = {};
  final double _cardWidth = 150.0;
  final double _cardHeight = 90.0;
  final double _horizontalSpacing = 150.0;
  final double _verticalSpacing = 120.0;

  late final TransformationController _transformationController;
  late final AnimationController _animationController;
  SpaceReorderDataModel? _draggedData;

  @override
  void initState() {
    _transformationController = TransformationController();
    _animationController = AnimationController(
      vsync: this,
      duration: const Duration(milliseconds: 150),
    );
    super.initState();
  }

  @override
  void didUpdateWidget(covariant CommunityStructureCanvas oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.selectedSpace?.uuid != oldWidget.selectedSpace?.uuid) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        if (mounted) {
          _animateToSpace(widget.selectedSpace);
        }
      });
    }
  }

  @override
  void dispose() {
    _transformationController.dispose();
    _animationController.dispose();
    super.dispose();
  }

  Set<String> _getAllDescendantUuids(SpaceModel space) {
    final uuids = <String>{};
    for (final child in space.children) {
      uuids.add(child.uuid);
      uuids.addAll(_getAllDescendantUuids(child));
    }
    return uuids;
  }

  void _runAnimation(Matrix4 target) {
    final animation = Matrix4Tween(
      begin: _transformationController.value,
      end: target,
    ).animate(_animationController);

    void listener() {
      _transformationController.value = animation.value;
    }

    animation.addListener(listener);
    _animationController.forward(from: 0).whenCompleteOrCancel(() {
      animation.removeListener(listener);
    });
  }

  void _animateToSpace(SpaceModel? space) {
    if (space == null) {
      _runAnimation(Matrix4.identity());
      return;
    }

    final position = _positions[space.uuid];
    if (position == null) return;

    const scale = 1;
    final viewSize = context.size;
    if (viewSize == null) return;

    final x = -position.dx * scale + (viewSize.width / 2) - (_cardWidth * scale / 2);
    final y =
        -position.dy * scale + (viewSize.height / 2) - (_cardHeight * scale / 2);

    final matrix = Matrix4.identity()
      ..translate(x, y)
      ..scale(scale);

    _runAnimation(matrix);
  }

  void _onReorder(SpaceReorderDataModel data, int newIndex) {
    final newCommunity = widget.community.copyWith();
    final children = data.parent?.children ?? newCommunity.spaces;
    final oldIndex = children.indexWhere((s) => s.uuid == data.space.uuid);
    if (oldIndex != -1) {
      final item = children.removeAt(oldIndex);
      if (newIndex > oldIndex) {
        children.insert(newIndex - 1, item);
      } else {
        children.insert(newIndex, item);
      }
    }
    context.read<CommunitiesBloc>().add(
          CommunitiesUpdateCommunity(newCommunity),
        );
  }

  void _onSpaceTapped(SpaceModel? space) {
    context.read<CommunitiesTreeSelectionBloc>().add(
          SelectSpaceEvent(community: widget.community, space: space),
        );
  }

  void _resetSelectionAndZoom([CommunityModel? community]) {
    context.read<CommunitiesTreeSelectionBloc>().add(
          SelectSpaceEvent(
            community: community ?? widget.community,
            space: null,
          ),
        );
  }

  void _calculateLayout(
    List<SpaceModel> spaces,
    int depth,
    Map<int, double> levelXOffset,
  ) {
    for (final space in spaces) {
      double childSubtreeWidth = 0;
      if (space.children.isNotEmpty) {
        _calculateLayout(space.children, depth + 1, levelXOffset);
        final firstChildPos = _positions[space.children.first.uuid];
        final lastChildPos = _positions[space.children.last.uuid];
        if (firstChildPos != null && lastChildPos != null) {
          childSubtreeWidth = (lastChildPos.dx + _cardWidth) - firstChildPos.dx;
        }
      }

      final currentX = levelXOffset.putIfAbsent(depth, () => 0.0);
      double? x;

      if (space.children.isNotEmpty) {
        final firstChildPos = _positions[space.children.first.uuid]!;
        x = firstChildPos.dx + (childSubtreeWidth - _cardWidth) / 2;
      } else {
        x = currentX;
      }

      if (x < currentX) {
        final shiftX = currentX - x;
        _shiftSubtree(space, shiftX);
        final keysToShift = levelXOffset.keys.where((d) => d > depth).toList();
        for (final key in keysToShift) {
          levelXOffset[key] = levelXOffset[key]! + shiftX;
        }
        x += shiftX;
      }

      final y = depth * (_verticalSpacing + _cardHeight);
      _positions[space.uuid] = Offset(x, y);
      levelXOffset[depth] = x + _cardWidth + _horizontalSpacing;
    }
  }

  void _shiftSubtree(SpaceModel space, double shiftX) {
    if (_positions.containsKey(space.uuid)) {
      _positions[space.uuid] = _positions[space.uuid]!.translate(shiftX, 0);
    }
    for (final child in space.children) {
      _shiftSubtree(child, shiftX);
    }
  }

  List<Widget> _buildTreeWidgets() {
    _positions.clear();
    final community = widget.community;

    final levelXOffset = <int, double>{};
    _calculateLayout(community.spaces, 0, levelXOffset);

    final selectedSpace = widget.selectedSpace;
    final highlightedUuids = <String>{};
    if (selectedSpace != null) {
      highlightedUuids.add(selectedSpace.uuid);
      highlightedUuids.addAll(_getAllDescendantUuids(selectedSpace));
    }

    final widgets = <Widget>[];
    final connections = <SpaceConnectionModel>[];
    _generateWidgets(
      widget.community.spaces,
      widgets,
      connections,
      highlightedUuids,
      community: widget.community,
    );

    final createButtonX = levelXOffset[0] ?? 0.0;
    const createButtonY = 0.0;

    widgets.add(
      Positioned(
        left: createButtonX,
        top: createButtonY,
        child: CreateSpaceButton(communityUuid: widget.community.uuid),
      ),
    );

    return [
      CustomPaint(
        painter: SpacesConnectionsArrowPainter(
          connections: connections,
          positions: _positions,
          highlightedUuids: highlightedUuids,
        ),
        child: Stack(alignment: AlignmentDirectional.center, children: widgets),
      ),
    ];
  }

  void _generateWidgets(
    List<SpaceModel> spaces,
    List<Widget> widgets,
    List<SpaceConnectionModel> connections,
    Set<String> highlightedUuids, {
    CommunityModel? community,
    SpaceModel? parent,
  }) {
    if (spaces.isNotEmpty) {
      final firstChildPos = _positions[spaces.first.uuid]!;
      final targetPos = Offset(
        firstChildPos.dx - (_horizontalSpacing / 4),
        firstChildPos.dy,
      );
      widgets.add(_buildDropTarget(parent, community, 0, targetPos));
    }

    for (var i = 0; i < spaces.length; i++) {
      final space = spaces[i];
      final position = _positions[space.uuid];
      if (position == null) {
        continue;
      }

      final isHighlighted = highlightedUuids.contains(space.uuid);
      final hasNoSelectedSpace = widget.selectedSpace == null;

      final spaceCard = SpaceCardWidget(
        buildSpaceContainer: () {
          return Opacity(
            opacity: hasNoSelectedSpace || isHighlighted ? 1.0 : 0.5,
            child: Tooltip(
              message: space.spaceName,
              preferBelow: false,
              child: SpaceCell(
                onTap: () => _onSpaceTapped(space),
                icon: space.icon,
                name: space.spaceName,
              ),
            ),
          );
        },
        onTap: () => SpaceDetailsDialogHelper.showCreate(
          context,
          communityUuid: widget.community.uuid,
        ),
      );

      final reorderData = SpaceReorderDataModel(
        space: space,
        parent: parent,
        community: community,
      );

      widgets.add(
        Positioned(
          left: position.dx,
          top: position.dy,
          width: _cardWidth,
          height: _cardHeight,
          child: Draggable<SpaceReorderDataModel>(
            data: reorderData,
            feedback: Material(
              color: Colors.transparent,
              child: Opacity(
                opacity: 0.2,
                child: SizedBox(
                  width: _cardWidth,
                  height: _cardHeight,
                  child: spaceCard,
                ),
              ),
            ),
            onDragStarted: () => setState(() => _draggedData = reorderData),
            onDragEnd: (_) => setState(() => _draggedData = null),
            onDraggableCanceled: (_, __) => setState(() => _draggedData = null),
            childWhenDragging: Opacity(opacity: 0.4, child: spaceCard),
            child: spaceCard,
          ),
        ),
      );

      final targetPos = Offset(
        position.dx + _cardWidth + (_horizontalSpacing / 4) - 20,
        position.dy,
      );
      widgets.add(_buildDropTarget(parent, community, i + 1, targetPos));

      for (final child in space.children) {
        connections.add(SpaceConnectionModel(from: space.uuid, to: child.uuid));
      }

      if (space.children.isNotEmpty) {
        _generateWidgets(
          space.children,
          widgets,
          connections,
          highlightedUuids,
          parent: space,
        );
      }
    }
  }

  Widget _buildDropTarget(
    SpaceModel? parent,
    CommunityModel? community,
    int index,
    Offset position,
  ) {
    return Positioned(
      left: position.dx,
      top: position.dy,
      width: 40,
      height: _cardHeight,
      child: DragTarget<SpaceReorderDataModel>(
        builder: (context, candidateData, rejectedData) {
          if (_draggedData == null) {
            return const SizedBox();
          }

          final isTargetForDragged = (_draggedData?.parent?.uuid == parent?.uuid &&
                  _draggedData?.community == null) ||
              (_draggedData?.community?.uuid == community?.uuid &&
                  _draggedData?.parent == null);

          if (!isTargetForDragged) {
            return const SizedBox();
          }

          return Container(
            width: 40,
            height: _cardHeight,
            decoration: BoxDecoration(
              color: context.theme.colorScheme.primary.withValues(
                alpha: candidateData.isNotEmpty ? 0.7 : 0.3,
              ),
              borderRadius: BorderRadius.circular(8),
            ),
            child: Icon(
              Icons.add,
              color: context.theme.colorScheme.onPrimary,
            ),
          );
        },
        onWillAcceptWithDetails: (data) {
          final children = parent?.children ?? community?.spaces ?? [];
          final isSameParent = (data.data.parent?.uuid == parent?.uuid &&
                  data.data.community == null) ||
              (data.data.community?.uuid == community?.uuid &&
                  data.data.parent == null);

          if (!isSameParent) {
            return false;
          }

          final oldIndex =
              children.indexWhere((s) => s.uuid == data.data.space.uuid);
          if (oldIndex == index || oldIndex == index - 1) {
            return false;
          }
          return true;
        },
        onAcceptWithDetails: (data) => _onReorder(data.data, index),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final treeWidgets = _buildTreeWidgets();
    return InteractiveViewer(
      transformationController: _transformationController,
      boundaryMargin: EdgeInsets.symmetric(
        horizontal: context.screenWidth * 0.3,
        vertical: context.screenHeight * 0.3,
      ),
      minScale: 0.5,
      maxScale: 3.0,
      constrained: false,
      child: GestureDetector(
        onTap: _resetSelectionAndZoom,
        child: SizedBox(
          width: context.screenWidth * 5,
          height: context.screenHeight * 5,
          child: Stack(children: treeWidgets),
        ),
      ),
    );
  }
}
