import 'package:flutter/material.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/products/domain/models/product.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/domain/models/space_details_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/presentation/widgets/space_details_action_buttons.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/domain/models/tag.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/presentation/widgets/add_device_type_widget.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/presentation/widgets/assign_tags_error_messages.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/presentation/widgets/assign_tags_table.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';
import 'package:uuid/uuid.dart';

class AssignTagsDialog extends StatefulWidget {
  const AssignTagsDialog({required this.space, super.key});

  final SpaceDetailsModel space;

  @override
  State<AssignTagsDialog> createState() => _AssignTagsDialogState();
}

class _AssignTagsDialogState extends State<AssignTagsDialog> {
  late SpaceDetailsModel _space;
  final Map<String, String> _validationErrors = {};

  @override
  void initState() {
    super.initState();
    _space = widget.space.copyWith(
      productAllocations:
          widget.space.productAllocations.map((e) => e.copyWith()).toList(),
      subspaces: widget.space.subspaces
          .map(
            (s) => s.copyWith(
              productAllocations:
                  s.productAllocations.map((e) => e.copyWith()).toList(),
            ),
          )
          .toList(),
    );
    _validateAllTags();
  }

  void _validateAllTags() {
    final newErrors = <String, String>{};
    final allAllocations = [
      ..._space.productAllocations,
      ..._space.subspaces.expand((s) => s.productAllocations),
    ];

    final allocationsByProductType = <String, List<ProductAllocation>>{};
    for (final allocation in allAllocations) {
      (allocationsByProductType[allocation.product.productType] ??= [])
          .add(allocation);
    }

    for (final productType in allocationsByProductType.keys) {
      final allocations = allocationsByProductType[productType]!;
      final tagCounts = <String, int>{};

      for (final allocation in allocations) {
        final tagName = allocation.tag.name.trim().toLowerCase();
        if (tagName.isEmpty) {
          newErrors[allocation.uuid] =
              'Tag for ${allocation.product.name} cannot be empty.';
        } else {
          tagCounts[tagName] = (tagCounts[tagName] ?? 0) + 1;
        }
      }

      for (final allocation in allocations) {
        final tagName = allocation.tag.name.trim().toLowerCase();
        if (tagName.isNotEmpty && (tagCounts[tagName] ?? 0) > 1) {
          newErrors[allocation.uuid] =
              'Tag "${allocation.tag.name}" is used by multiple $productType devices.';
        }
      }
    }

    setState(() {
      _validationErrors
        ..clear()
        ..addAll(newErrors);
    });
  }

  void _handleTagChange(String allocationUuid, Tag newTag) {
    setState(() {
      var index =
          _space.productAllocations.indexWhere((pa) => pa.uuid == allocationUuid);
      if (index != -1) {
        final allocation = _space.productAllocations[index];
        _space.productAllocations[index] = allocation.copyWith(tag: newTag);
      } else {
        for (final subspace in _space.subspaces) {
          index = subspace.productAllocations
              .indexWhere((pa) => pa.uuid == allocationUuid);
          if (index != -1) {
            final allocation = subspace.productAllocations[index];
            subspace.productAllocations[index] = allocation.copyWith(tag: newTag);
            break;
          }
        }
      }
    });
    _validateAllTags();
  }

  void _handleLocationChange(String allocationUuid, String? newSubspaceUuid) {
    setState(() {
      ProductAllocation? allocationToMove;

      var index =
          _space.productAllocations.indexWhere((pa) => pa.uuid == allocationUuid);
      if (index != -1) {
        allocationToMove = _space.productAllocations.removeAt(index);
      } else {
        for (final subspace in _space.subspaces) {
          index = subspace.productAllocations
              .indexWhere((pa) => pa.uuid == allocationUuid);
          if (index != -1) {
            allocationToMove = subspace.productAllocations.removeAt(index);
            break;
          }
        }
      }

      if (allocationToMove == null) return;

      if (newSubspaceUuid == null) {
        _space.productAllocations.add(allocationToMove);
      } else {
        _space.subspaces
            .firstWhere((s) => s.uuid == newSubspaceUuid)
            .productAllocations
            .add(allocationToMove);
      }
    });
  }

  void _handleProductDelete(String allocationUuid) {
    setState(() {
      _space.productAllocations.removeWhere((pa) => pa.uuid == allocationUuid);

      for (final subspace in _space.subspaces) {
        subspace.productAllocations.removeWhere(
          (pa) => pa.uuid == allocationUuid,
        );
      }
    });
    _validateAllTags();
  }

  @override
  Widget build(BuildContext context) {
    final allProductAllocations = [
      ..._space.productAllocations,
      ..._space.subspaces.expand((s) => s.productAllocations),
    ];

    final productLocations = <String, String?>{};
    for (final pa in _space.productAllocations) {
      productLocations[pa.uuid] = null;
    }
    for (final subspace in _space.subspaces) {
      for (final pa in subspace.productAllocations) {
        productLocations[pa.uuid] = subspace.uuid;
      }
    }

    final hasErrors = _validationErrors.isNotEmpty;

    return AlertDialog(
      title: const SelectableText('Assign Tags'),
      content: ConstrainedBox(
        constraints: BoxConstraints(
          maxWidth: context.screenWidth * 0.6,
          minWidth: context.screenWidth * 0.6,
          maxHeight: context.screenHeight * 0.8,
        ),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            SizedBox(
              width: double.infinity,
              child: AssignTagsTable(
                productAllocations: allProductAllocations,
                subspaces: _space.subspaces,
                productLocations: productLocations,
                onTagSelected: _handleTagChange,
                onLocationSelected: _handleLocationChange,
                onProductDeleted: _handleProductDelete,
              ),
            ),
            if (hasErrors)
              AssignTagsErrorMessages(
                errorMessages: _validationErrors.values.toSet().toList(),
              ),
          ],
        ),
      ),
      actions: [
        SpaceDetailsActionButtons(
          onSave: hasErrors ? null : () => Navigator.of(context).pop(_space),
          onCancel: () async {
            final newProducts = await showDialog<List<Product>>(
              context: context,
              builder: (context) => const AddDeviceTypeWidget(),
            );

            if (newProducts == null || newProducts.isEmpty) return;

            setState(() {
              for (final product in newProducts) {
                _space.productAllocations.add(
                  ProductAllocation(
                    uuid: const Uuid().v4(),
                    product: product,
                    tag: Tag.empty(),
                  ),
                );
              }
            });
            _validateAllTags();
          },
          cancelButtonLabel: 'Add New Device',
        )
      ],
    );
  }
}
