import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/common/dialog_dropdown.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/domain/models/space_details_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/data/services/remote_tags_service.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/domain/models/tag.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/presentation/bloc/tags_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/presentation/widgets/product_tag_field.dart';
import 'package:syncrow_web/services/api/http_service.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

class AssignTagsTable extends StatelessWidget {
  const AssignTagsTable({
    required this.productAllocations,
    required this.subspaces,
    required this.productLocations,
    required this.onTagSelected,
    required this.onLocationSelected,
    required this.onProductDeleted,
    super.key,
  });

  final List<ProductAllocation> productAllocations;
  final List<Subspace> subspaces;
  final Map<String, String?> productLocations;
  final void Function(String, Tag) onTagSelected;
  final void Function(String, String?) onLocationSelected;
  final void Function(String) onProductDeleted;

  DataColumn _buildDataColumn(BuildContext context, String label) {
    return DataColumn(
      label: SelectableText(label, style: context.textTheme.bodyMedium),
    );
  }

  @override
  Widget build(BuildContext context) {
    return BlocProvider<TagsBloc>(
      create: (BuildContext context) => TagsBloc(
        RemoteTagsService(HTTPService()),
      )..add(const LoadTags()),
      child: BlocBuilder<TagsBloc, TagsState>(
        builder: (context, state) {
          return switch (state) {
            TagsLoading() || TagsInitial() => const Center(
                child: CircularProgressIndicator(),
              ),
            TagsFailure(:final message) => Center(
                child: Text(message),
              ),
            TagsLoaded(:final tags) => ClipRRect(
                borderRadius: BorderRadius.circular(20),
                child: DataTable(
                  headingRowColor: WidgetStateProperty.all(
                    ColorsManager.dataHeaderGrey,
                  ),
                  key: ValueKey(productAllocations.length),
                  border: TableBorder.all(
                    color: ColorsManager.dataHeaderGrey,
                    width: 1,
                    borderRadius: BorderRadius.circular(20),
                  ),
                  columns: [
                    _buildDataColumn(context, '#'),
                    _buildDataColumn(context, 'Device'),
                    _buildDataColumn(context, 'Tag'),
                    _buildDataColumn(context, 'Location'),
                  ],
                  rows: productAllocations.isEmpty
                      ? [
                          DataRow(
                            cells: [
                              DataCell(
                                Center(
                                  child: SelectableText(
                                    'No Devices Available',
                                    style: context.textTheme.bodyMedium?.copyWith(
                                      color: ColorsManager.lightGrayColor,
                                    ),
                                  ),
                                ),
                              ),
                              DataCell.empty,
                              DataCell.empty,
                              DataCell.empty,
                            ],
                          ),
                        ]
                      : List.generate(productAllocations.length, (index) {
                          final productAllocation = productAllocations[index];
                          final allocationUuid = productAllocation.uuid;

                          final availableTags = tags
                              .where(
                                (tag) =>
                                    !productAllocations
                                        .where((p) =>
                                            p.product.productType ==
                                            productAllocation.product.productType)
                                        .map((p) => p.tag.name.toLowerCase())
                                        .contains(tag.name.toLowerCase()) ||
                                    tag.uuid == productAllocation.tag.uuid,
                              )
                              .toList();

                          final currentLocationUuid =
                              productLocations[allocationUuid];
                          final currentLocationName = currentLocationUuid == null
                              ? 'Main Space'
                              : subspaces
                                  .firstWhere((s) => s.uuid == currentLocationUuid)
                                  .name;

                          return DataRow(
                            key: ValueKey(allocationUuid),
                            cells: [
                              DataCell(Text((index + 1).toString())),
                              DataCell(
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                  children: [
                                    Expanded(
                                        child: Text(
                                      productAllocation.product.name,
                                      overflow: TextOverflow.ellipsis,
                                    )),
                                    const SizedBox(width: 10),
                                    Container(
                                      width: 20,
                                      height: 20,
                                      decoration: BoxDecoration(
                                        shape: BoxShape.circle,
                                        border: Border.all(
                                          color: ColorsManager.lightGrayColor,
                                          width: 1,
                                        ),
                                      ),
                                      child: IconButton(
                                        icon: const Icon(
                                          Icons.close,
                                          color: ColorsManager.lightGreyColor,
                                          size: 16,
                                        ),
                                        onPressed: () {
                                          onProductDeleted(allocationUuid);
                                        },
                                        tooltip: 'Delete Tag',
                                        padding: EdgeInsets.zero,
                                        constraints: const BoxConstraints(),
                                      ),
                                    ),
                                  ],
                                ),
                              ),
                              DataCell(
                                Container(
                                  alignment: Alignment.centerLeft,
                                  width: double.infinity,
                                  child: ProductTagField(
                                    key: ValueKey('dropdown_$allocationUuid'),
                                    productName: productAllocation.product.uuid,
                                    initialValue: productAllocation.tag,
                                    onSelected: (newTag) {
                                      onTagSelected(allocationUuid, newTag);
                                    },
                                    items: availableTags,
                                  ),
                                ),
                              ),
                              DataCell(
                                SizedBox(
                                    width: double.infinity,
                                    child: DialogDropdown(
                                      items: [
                                        'Main Space',
                                        ...subspaces.map((s) => s.name)
                                      ],
                                      selectedValue: currentLocationName,
                                      onSelected: (newLocationName) {
                                        final newSubspaceUuid = newLocationName ==
                                                'Main Space'
                                            ? null
                                            : subspaces
                                                .firstWhere(
                                                    (s) => s.name == newLocationName)
                                                .uuid;
                                        onLocationSelected(
                                            allocationUuid, newSubspaceUuid);
                                      },
                                    )),
                              ),
                            ],
                          );
                        }),
                ),
              ),
            _ => const SizedBox.shrink(),
          };
        },
      ),
    );
  }
}
